%% BYOM, byom_guts_constant_FLU_validation.m, full GUTS model
%
% * Author: Tjalling Jager 
% * Date: November 2023
% * Web support: <http://www.debtox.info/byom.html>
%
% BYOM is a General framework for simulating model systems. The files in
% this directory use an analytical solution only, and therefore
% <derivatives.html derivatives.m> will be missing.
%
% *The model:* fitting survival data with the
% <http://www.debtox.info/about_guts.html GUTS> special cases based on the
% full model (TK and damage dynamics separate): SD, IT and mixed (or GUTS
% proper). The standard full GUTS model package was adapted slightly to
% allow the double-scaled model version, and to easily switch from the
% script file between the full model and the reduced model (using
% glo.fastkin). 
% 
% *This script:* data set for FLU in _Daphnia magna_ from: Tolosi and De
% Liguoro (2021), <https://doi.org/10.1016/j.ecoenv.2021.112778>. This
% script allows to redo the validations from my manuscript on delayed
% toxicity (in prep.): predictions for the recovery period based on the
% data for 2-days exposure. The data points were extracted from the original
% publication using <https://github.com/jornbr/plotreader/wiki PlotReader>,
% so there may be small errors (particularly where treatments overlap in
% low-level response).
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(1) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 3; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. 

% observed number of survivors, time in days, conc in mg/L
DATA{1} = [-1	0	0.7	1.2	2.1	3.8	6.9	12.3	22.2	40
0	20	20	20	20	20	20	20	20	20
1	20	20	20	20	20	19	19	16	9
2	20	20	20	20	20	19	18	14	3
4	19	20	20	20	19	19	18	12	3
6	19	19	18	18	17	13	7	0	0
8	19	19	18	18	17	12	6	0	0
10	18	19	18	18	17	10	3	0	0
12	18	19	18	18	17	9	3	0	0];
 
% Define the exposure scenarios as a series of pulses with a new constant
% exposure in each time period. This allow use of an analytical solution
% for the TK/damage dynamics in simplefun.
Cw = [0 0.7	1.2	2.1	3.8	6.9	12.3	22.2	40
    0 0.7	1.2	2.1	3.8	6.9	12.3	22.2	40
    2 0       0   0   0   0    0       0     0];
 
make_scen(2,Cw); % create the globals to define the forcing function

DATA{2} = []; % internal concentrations are not provided

% scaled damage; there are no data for this stage, but dummy data are added
% to make sure that the damage prediction can be plotted by calc_and_plot
% (without having to plot ALL model curves in ALL sub-plots)
DATA{3} = [1 DATA{1}(1,2:end)
           0  1e-6     1e-6    1e-6    1e-6   1e-6   1e-6    1e-6   1e-6   1e-6]; 
       
glo.wts = [1 1 0];  % set zero weight to fake data for scaled damage

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,~,par] = select_pred([1 1 2]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters in par_saved, as well as all settings used to
% generate the MAT file (in glo) apart from the exposure profile (which we
% have already defined above). This only works when the MAT file was
% generated by BYOM v.6.0 BETA 7 or newer as older versions won't save glo.

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios.

X0mat      = []; % clear the one loaded from the saved file
X0mat(1,:) = unique(DATA{1}(1,2:end)); % scenarios (concentrations or identifiers)
X0mat(2,:) = 1; % initial survival probability
X0mat(3,:) = 0; % initial (scaled) internal concentration
X0mat(4,:) = 0; % initial scaled damage

%% Initial values for the model parameters
% Model parameters are part of a 'structure' for easy reference. Note that
% par and glo are read from file, so there's nothing to do here.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculations and plotting
% Here, the function is called that will do the calculation and the
% plotting. The files in this directory always apply the analytical
% solution for damage in simplefun.

opt_optim.type   = 4; % optimisation method 1) simplex, 4 parameter-space explorer
opt_optim.fit    = 0; % fit the parameters (1), or don't (0)
opt_plot.bw      = 1; % if set to 1, plots in black and white with different plot symbols
opt_plot.annot   = 2; % annotations in multiplot for fits: 1) box with parameter estimates 2) single legend
opt_plot.statsup = [glo.locC glo.locD]; % vector with states to suppress in plotting fits

% optimise and plot (fitted parameters in par_out)
par_out = calc_optim(par,opt_optim); % start the optimisation
calc_and_plot(par_out,opt_plot); % calculate model lines and plot them

opt_conf.type    = 3; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
opt_conf.lim_set = 2; % for lik-region sample: use limited set of n_lim points (1) or outer hull (2) to create CIs
opt_tktd.max_exp = 0; % set to 1 to maximise exposure/damage plots on exposure rather than damage
opt_tktd.obspred = 0; % plot predicted-observed plots (1) or not (0)
opt_tktd.statsup = [glo.locC glo.locD]; % states to suppress from the plots (e.g., locS)
opt_tktd.notitle  = 1; % set to 1 to suppress titles above plots

h_coll = plot_tktd(par_out,opt_tktd,opt_conf); % leaving opt_conf empty suppresses all CIs for these plots

% resize the multipanel plot for survival and all treatments to a more
% practical size for in the supporting information
figh = h_coll{1};
figh.Position(3) = 2000;
figh.Position(4) = 250;

% overwrite the previous plot made by plot_tktd with the resized one
savenm = ['tktd_plot_',glo.basenm];%
save_plot(figh,savenm);

