%% BYOM, byom_guts_showcal.m
%
% * Author: Tjalling Jager
% * Date: November 2021
% * Web support: <http://www.debtox.info/byom.html>
% * Back to index <walkthrough_guts.html>
%
% BYOM is a General framework for simulating model systems. The files in
% this directory use an analytical solution only, and therefore
% <derivatives.html derivatives.m> will be missing.
%
% *The model:* fitting survival data with the
% <http://www.debtox.info/about_guts.html GUTS> special cases based on the
% reduced model (TK and damage dynamics lumped): SD, IT and mixed (or GUTS
% proper). 
%
% *This script:* Show results from a previous calibration. All settings are
% loaded from MAT file.
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(0) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% The data set will be loaded from the MAT file by select_pred. This
% requires that DATA was saved in the calibration script!

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,fname_prof,par] = select_pred([3 1 1]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters, as well as all settings used to generate the MAT
% file (in glo) apart from the exposure profile. This only works when the
% MAT file was generated by BYOM v.6.0 BETA 7 or newer as older versions
% won't save glo. Also, the data set is loaded, when it was saved in the
% calibration file with the same name (..._DATA.mat).
% 
% The parameter structure <par> is loaded from the selected MAT file. Even
% though it does not need to be used to call other functions, it must be
% specified for the code to run error free.

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios. All identifiers and initial values, as part of
% the matrix X0mat, are also loaded by select_pred already. Feel free to
% overwrite them with other values if needed.
% 
% NOTE: the glo.locD etc. have all been loaded from the MAT file by select_pred.

%% Initial values for the model parameters
% All parameters have been loaded from file using select_pred. All global
% parameters, as part of the structure glo, are also loaded by
% select_pred already. Feel free to overwrite them with other values.

%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% used, based on the data set

% All plot labels have been loaded from file using select_pred.
% Vector glo.t is already defined in select_pred from the selected profile.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculations and plotting
% Here, the function is called that will do the calculation and the
% plotting. Options for the plotting can be set using opt_plot (see
% prelim_checks.m). Options for the optimisation routine can be set using
% opt_optim. The files in this directory always apply the analytical
% solution for damage in simplefun.

% These settings for opt_optim allow you to see the parspace plot, without refitting.
if conf_type == 3 % only when the saved set is for the parspace explorer
    opt_optim.type     = 4; % optimisation method 1) simplex, 4 parameter-space explorer
    opt_optim.fit      = 1; % fit the parameters (1), or don't (0)
    opt_optim.ps_saved = 1; % use saved set for parameter-space explorer (1) or not (0);
    % use calc_optim to display parameter-space plot and return fitted parameters in par_out
    par_out = calc_optim(par,opt_optim); % start the optimisation

    opt_plot.bw      = 1; % if set to 1, plots in black and white with different plot symbols
    opt_plot.annot   = 2; % annotations in multiplot for fits: 1) box with parameter estimates 2) single legend

    calc_and_plot(par_out,opt_plot); % calculate model lines and plot them
end

%% Plot results with confidence intervals
% The following code can be used to make plots with confidence intervals.
% Options for confidence bounds on model curves can be set using opt_conf
% (see prelim_checks). The plot_tktd function makes multiplots for the
% survival data, which are more readable when plotting with various
% intervals.

opt_conf.type    = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
opt_conf.lim_set = 2; % for lik-region sample: use limited set of n_lim points (1) or outer hull (2) to create CIs
opt_tktd.notitle = 0; % set to 1 to suppress titles above plots (becomes messy for GUTS)

plot_tktd([],opt_tktd,opt_conf); 
% Note that first input is left empty: this is for the parameter structure,
% which is then obtained from the saved sample. When leaving the options
% opt_conf empty, it suppresses all CIs for these plots. However, in that
% case, par would need to be specified.
